package eu.maydu.gwt.validation.client.validators.datetime;

import com.google.gwt.user.client.ui.HasValue;

import eu.maydu.gwt.validation.client.ValidationAction;
import eu.maydu.gwt.validation.client.ValidationResult;
import eu.maydu.gwt.validation.client.i18n.StandardValidationMessages;
import eu.maydu.gwt.validation.client.i18n.ValidationMessages;
import eu.maydu.gwt.validation.client.validators.StringValidator;
import eu.maydu.gwt.validation.client.validators.ValidatorAlgorithmResult;
import eu.maydu.gwt.validation.client.validators.datetime.algorithms.TimeValidatorAlgorithm;


/**
 * This validator validates that a given input is a time of day.
 * 
 * Currently it only validates the european 24 hour format with
 * seconds optionally (This is configurable).
 * 
 * 
 * @author Anatol Mayen
 *
 */
public class TimeValidator<V extends HasValue<String>> extends StringValidator<TimeValidator<V>, V> {

	private TimeValidatorAlgorithm algorithm;
	
	public TimeValidator(V text) {
		this(text, false, false);
	}
	
	public TimeValidator(V text, String customMessageKey) {
		this(text, false, false, customMessageKey);
	}
	
	/**
	 * 
	 * @param text The textbox that contains the user input
	 * @param secondsAllowed Should seconds be optional
	 * @param secondsRequired Are seconds required (overwrites the secondsAllowed field)
	 */
	public TimeValidator(V text, boolean secondsAllowed, boolean secondsRequired) {
		super(text);
		algorithm = new TimeValidatorAlgorithm(secondsAllowed, secondsRequired);
	}
	
	public TimeValidator(V text, boolean secondsAllowed, boolean secondsRequired, String customMessageKey) {
		super(text);
		this.setCustomMsgKey(customMessageKey);
		algorithm = new TimeValidatorAlgorithm(secondsAllowed, secondsRequired);
	}
		
	@Override
	public void invokeActions(ValidationResult result) {
		for(ValidationAction<V> va : this.getFailureActions()) {
			va.invoke(result, valueHolder);
		}
	}

	@Override
	public <M extends ValidationMessages> ValidationResult validate(M messages) {
		String input = valueHolder.getValue();
		
		if(!this.isRequired() && input.equals(""))
			return null;
		
		ValidatorAlgorithmResult result = algorithm.validate(input);
		
		if(result == null)
			return null;
		
		String stdMessage = getStdMessage(messages.getStandardMessages(), result.getErrorCode(), result.getParameters()[0].toString());
		
		return new ValidationResult(this.getErrorMessage(messages, stdMessage, result.getParameters()));
		
	}

	private String getStdMessage(StandardValidationMessages stdMessages,
			int errorCode, String parameter) {
		switch(errorCode) {
		case TimeValidatorAlgorithm.NOT_A_VALID_TIME_WITHOUT_SECONDS:
			return stdMessages.notAValidTimeWithoutSeconds(parameter);
		case TimeValidatorAlgorithm.NOT_A_VALID_TIME_WITH_SECONDS_OPTIONALLY:
			return stdMessages.notAValidTimeWithSecondsOptionally(parameter);
		case TimeValidatorAlgorithm.NOT_A_VALID_TIME_WITH_SECONDS_REQUIRED:
			return stdMessages.notAValidTimeWithSecondsRequired(parameter);
		}
		
		return null;
	}

}
